@@ -36,12 +36,17 @@
     OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
     OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
+
+
+/* Note: this file is a wrapper for the R language. It doesn't need to be compiled
+   if wrapping it for a different language. */
+#ifdef _FOR_R
+
 #include <R_ext/Rdynload.h>
 #include <R.h>
 #include <Rinternals.h>
-#ifndef _FOR_R
-#define _FOR_R
-#endif
+#include <limits.h>
+#include <math.h>
 #include "poismf.h"
 
 /* FORTRAN-BLAS -> CBLAS */
@@ -87,9 +92,14 @@ SEXP wrapper_run_poismf
     SEXP method, SEXP limit_step,
     SEXP l2_reg, SEXP l1_reg,
     SEXP w_mult, SEXP step_size,
-    SEXP niter, SEXP maxupd, SEXP nthreads
+    SEXP niter, SEXP maxupd,
+    SEXP handle_interrupt, SEXP nthreads
 )
 {
+    if (Rf_xlength(Xr) == 0) {
+        Rf_error("'X' contains no non-zero entries.");
+        return R_NilValue;
+    }
     int ret_code = run_poismf(
         REAL(A), REAL(Xr), INTEGER(Xr_indptr), INTEGER(Xr_indices),
         REAL(B), REAL(Xc), INTEGER(Xc_indptr), INTEGER(Xc_indices),
@@ -98,7 +108,7 @@ SEXP wrapper_run_poismf
         Rf_asReal(w_mult), Rf_asReal(step_size),
         (Method) Rf_asInteger(method), (bool) Rf_asLogical(limit_step),
         (size_t) Rf_asInteger(niter), (size_t) Rf_asInteger(maxupd),
-        Rf_asInteger(nthreads)
+        (bool) Rf_asLogical(handle_interrupt), Rf_asInteger(nthreads)
     );
     if (ret_code == 1) Rf_error("Out of memory.");
     return R_NilValue;
@@ -111,7 +121,7 @@ SEXP wrapper_predict_multiple
     SEXP nthreads
 )
 {
-    size_t nnz = (size_t) Rf_length(ixA);
+    size_t nnz = (size_t) Rf_xlength(ixA);
     SEXP out = PROTECT(Rf_allocVector(REALSXP, nnz));
     predict_multiple(
         REAL(out), REAL(A), REAL(B),
@@ -135,14 +145,14 @@ SEXP wrapper_predict_factors
 )
 {
     size_t k_szt = (size_t) Rf_asInteger(k);
-    size_t dimA = (size_t)Rf_length(A_old) / k_szt;
+    size_t dimA = (size_t)Rf_xlength(A_old) / k_szt;
     SEXP out = PROTECT(Rf_allocVector(REALSXP, k_szt));
 
     int ret_code = factors_single(
         REAL(out), k_szt,
         REAL(A_old), dimA,
         REAL(counts), INTEGER(ix),
-        (sparse_ix) Rf_length(counts),
+        (size_t) Rf_xlength(counts),
         REAL(B), REAL(Bsum),
         Rf_asInteger(maxupd),
         Rf_asReal(l2_reg),
@@ -168,8 +178,13 @@ SEXP wrapper_predict_factors_multiple
     SEXP method, SEXP limit_step, SEXP nthreads
 )
 {
-    SEXP out = PROTECT(Rf_allocVector(REALSXP, (size_t)Rf_asInteger(k)
-                                                * (size_t)Rf_asInteger(dimA)));
+    if ((R_xlen_t)Rf_asInteger(k) * (R_xlen_t)Rf_asInteger(dimA) <= 0)
+    {
+        Rf_error("Requested array dimensions exceed R limits.");
+        return R_NilValue; /* not reached */
+    }
+    SEXP out = PROTECT(Rf_allocVector(REALSXP, (R_xlen_t)Rf_asInteger(k)
+                                                * (R_xlen_t)Rf_asInteger(dimA)));
 
     int res = factors_multiple(
         REAL(out), REAL(B), REAL(A_old), REAL(Bsum),
@@ -202,7 +217,7 @@ SEXP wrapper_eval_llk
     long double llk = eval_llk(
         REAL(A), REAL(B),
         INTEGER(ixA), INTEGER(ixB), REAL(Xcoo),
-        (size_t) Rf_length(Xcoo), Rf_asInteger(k),
+        (size_t) Rf_xlength(Xcoo), Rf_asInteger(k),
         (bool) Rf_asLogical(full_llk), (bool) Rf_asLogical(include_missing),
         (size_t) Rf_asInteger(dimA), (size_t) Rf_asInteger(dimB),
         Rf_asInteger(nthreads)
@@ -222,14 +237,14 @@ SEXP wrapper_topN
     SEXP top_n, SEXP nthreads
 )
 {
-    size_t n_include = (size_t) Rf_length(include_ix);
-    size_t n_exclude = (size_t) Rf_length(exclude_ix);
+    size_t n_include = (size_t) Rf_xlength(include_ix);
+    size_t n_exclude = (size_t) Rf_xlength(exclude_ix);
     int res = topN(
-        REAL(a_vec), REAL(B), Rf_length(a_vec),
+        REAL(a_vec), REAL(B), Rf_xlength(a_vec),
         n_include? INTEGER(include_ix) : (int*)NULL, n_include,
         n_exclude? INTEGER(exclude_ix) : (int*)NULL, n_exclude,
         INTEGER(outp_ix),
-        (Rf_length(outp_score) > 0)?
+        (Rf_xlength(outp_score) > 0)?
             REAL(outp_score) : (double*)NULL,
         (size_t) Rf_asInteger(top_n), (size_t) Rf_asInteger(dimB),
         Rf_asInteger(nthreads)
@@ -241,15 +256,51 @@ SEXP wrapper_topN
     return R_NilValue;
 }
 
+SEXP check_size_below_int_max
+(
+    SEXP dim1, SEXP dim2
+)
+{
+    SEXP out = PROTECT(Rf_allocVector(LGLSXP, 1));
+    LOGICAL(out)[0] = (size_t)Rf_asInteger(dim1) * (size_t)Rf_asInteger(dim2) <= INT_MAX;
+    UNPROTECT(1);
+    return out;
+}
+
+SEXP large_rnd_vec
+(
+    SEXP dim1, SEXP dim2,
+    SEXP do_gamma
+)
+{
+    R_xlen_t tot_size = (size_t)Rf_asInteger(dim1) * (size_t)Rf_asInteger(dim2);
+    if (tot_size < 0) {
+        Rf_error("Requested array dimensions exceed R limits.");
+        return Rf_allocVector(REALSXP, 0);
+    }
+    SEXP out = PROTECT(Rf_allocVector(REALSXP, tot_size));
+    double *restrict ptr_vec = REAL(out);
+    GetRNGstate();
+    for (size_t ix = 0; ix < (size_t)tot_size; ix++)
+        ptr_vec[ix] = unif_rand();
+    PutRNGstate();
+    if (Rf_asLogical(do_gamma))
+        for (size_t ix = 0; ix < (size_t)tot_size; ix++)
+            ptr_vec[ix] = -log(ptr_vec[ix]);
+    UNPROTECT(1);
+    return out;
+}
 
 
 static const R_CallMethodDef callMethods [] = {
-    {"wrapper_run_poismf", (DL_FUNC) &wrapper_run_poismf, 20},
+    {"wrapper_run_poismf", (DL_FUNC) &wrapper_run_poismf, 21},
     {"wrapper_predict_multiple", (DL_FUNC) &wrapper_predict_multiple, 6},
     {"wrapper_predict_factors", (DL_FUNC) &wrapper_predict_factors, 11},
     {"wrapper_predict_factors_multiple", (DL_FUNC) &wrapper_predict_factors_multiple, 16},
     {"wrapper_eval_llk", (DL_FUNC) &wrapper_eval_llk, 11},
     {"wrapper_topN", (DL_FUNC) &wrapper_topN, 9},
+    {"check_size_below_int_max", (DL_FUNC) &check_size_below_int_max, 2},
+    {"large_rnd_vec", (DL_FUNC) &large_rnd_vec, 3},
     {NULL, NULL, 0}
 }; 
 
@@ -259,3 +310,4 @@ void R_init_poismf(DllInfo *info)
     R_useDynamicSymbols(info, TRUE);
 }
 
+#endif /* _FOR_R */
